--[[
 Equipment Companion module for MiniPet
 Version 5.1.219
 $Revision: 183 $

 All credit for authoring this addon is listed below
]]--

--[[
Copyright (c) 2010, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local L = LibStub( "AceLocale-3.0" ):GetLocale( "MiniPet equipment Module" );

local LibLordFarlander = LibStub( "LibLordFarlander-2.0", true );
if( not LibLordFarlander ) then
    return;
end--if

local PetAndMountDatabase = LibStub( "LibLordFarlander-PetAndMountDatabase-1.1", true );
if( not PetAndMountDatabase ) then
    return;
end--if

local LibLordFarlander_UI = LibStub( "LibLordFarlander-UI-2.0", true );
if( not LibLordFarlander_UI ) then
    return;
end--if

local TR = LibStub( "LibLordFarlander-TableRecycler-1.0", true );
if( not TR ) then
    return;
end--if

if( not LibStub( "LibLordFarlander-Randomizer-2.0", true ) ) then
    return;
end--if

local Equipment = {
    displayname = L["Equipment"],
    settings = {
        [L["equipmentCheck"]] = {
            type = "toggle",
            name = L["Check to make sure that equipment companions are equipped"],
            width = "double",                     
            arg = {
                DescriptionString = L["Toggles whether companions summoned by equipment will be suggested when the item is unequipped."],
                VariableName = "EquipmentCheck",
                Default = false,
            },
            desc = LibLordFarlander_UI.BooleanOptionDescription,
            get = LibLordFarlander_UI.GetGenericOption,
            set = "SetOption_ResetPets",                       
        },        
    },
    events = {
        "UNIT_INVENTORY_CHANGED",
        "CheckItem",
        "ScanPlayer",
    },
};

function Equipment:OnUNIT_INVENTORY_CHANGED( MiniPet, unit )
    if( unit == "player" ) then
        MiniPet:DirtyBag( -254 );
    end--if
end--Equipment:OnUNIT_INVENTORY_CHANGED( MiniPet, _, unit )

function Equipment:OnCheckItem( MiniPet, itemID, bag, slot )
    return ( not MiniPet:GetSetting( "profile", "EquipmentCheck" ) ) or ( not PetAndMountDatabase:ItemInSet( itemID, "Critters.Equipment" ) );
end--Equipment:OnCheckItem( MiniPet, itemID, bag, slot )

function Equipment:OnScanPlayer( MiniPet, dirtyBags, firstSeen )
    local returnVal = false;
    local opEquipmentCheck = MiniPet:GetSetting( "profile", "EquipmentCheck" );
    
    if( dirtyBags[-254] ) then
        MiniPet:DoCustomEvent( "StartBagScan", -254 );

        local iterator, handle = PetAndMountDatabase:IterateSet( "Critters.Equipment" );
        local pet, equipmentSlot = iterator( handle );
        local GetItemIDFromLink = LibLordFarlander.GetItemIDFromLink;
        local curTime = time();
    
        while pet do
            equipmentSlot = GetInventorySlotInfo( equipmentSlot );
            
            local itemLink = GetInventoryItemLink( "player", equipmentSlot );

            if( itemLink and ( GetItemIDFromLink( itemLink ) == pet ) ) then
                if( not firstSeen[pet] ) then
                    firstSeen[pet] = time();
                end--if
                MiniPet:DoCustomEvent( "AddInventoryItem", equipmentSlot );
                returnVal = true;
            end--if
            pet, equipmentSlot = iterator( handle );
        end--for
        MiniPet:DoCustomEvent( "FinalizeBag", -254 );
    end--if
    return returnVal;
end--Equipment:ScanPlayer( MiniPet, dirtyBags, firstSeen )

LibStub( "LibLordFarlander-Randomizer-2.0" ):RegisterModuleToHandler( "MiniPet", "Equipment", Equipment, "Companions" );
