--[[
 MiniPet Addon
 Version 5.1.219
 $Revision: 219 $

 All credit for authoring this addon is listed below

 Visit WarcraftPets.com for all your minipet needs!
]]--

--[[
    MiniPet by Carra of Stormrage

    There are a number of events where the player looses his pet and we can notify him:
    -Entering the game                   >OK
    -Entering an instance                >OK
    -Dieing and getting back alive       >Usually OK :p
    -Using a Taxi Service                >OK
    -Teleporting/Summoning/Hearthstoning >OK but no summoning

    Based originally on MiniPetLeash (completely rewritten except for the events which are copied).

    Updated and converted to Ace3 by LordFarlander
]]--

--[[
Copyright (c) 2008-2010, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local L = LibStub( "AceLocale-3.0" ):GetLocale( "MiniPet" );

_G["BINDING_NAME_CLICK MiniPet_Button:LeftButton"] = L["Summon Companion"];
BINDING_HEADER_MINIPETHEADER = L["MiniPet"];

local LibLordFarlander = LibStub( "LibLordFarlander-2.0", true );
if( not LibLordFarlander ) then
    error( L["%s requires %s"]:format( L["MiniPet"], "LibLordFarlander-2.0" ) );
    return;
end--if

local ItemList = LibStub( "LibLordFarlander-ItemList-2.0", true );
if( not ItemList ) then
    error( L["%s requires %s"]:format( L["MiniPet"], "LibLordFarlander-ItemList-2.0" ) );
    return;
end--if

local LibLordFarlander_UI = LibStub( "LibLordFarlander-UI-2.0", true );
if( not LibLordFarlander_UI ) then
    error( L["%s requires %s"]:format( L["MiniPet"], "LibLordFarlander-UI-2.0" ) );
    return;
end--if

local PetAndMountDatabase = LibStub( "LibLordFarlander-PetAndMountDatabase-1.1", true );
if( not PetAndMountDatabase ) then
    error( L["%s requires %s"]:format( L["MiniPet"], "LibLordFarlander-PetAndMountDatabase-1.1" ) );
    return;
end--if

local TR = LibStub( "LibLordFarlander-TableRecycler-1.0", true );
if( not TR ) then
    error( L["%s requires %s"]:format( L["MiniPet"], "LibLordFarlander-TableRecycler-1.0" ) );
    return;
end--if

MiniPet = LibStub( "AceAddon-3.0" ):NewAddon( "MiniPet", "AceEvent-3.0", "AceTimer-3.0", "AceConsole-3.0", "LibLordFarlander-Randomizer-2.0" );
MiniPet.version = { major = 5, minor = 1, revision = tonumber( "219" ) };

--Global Values
MiniPet.DEBUG = false;            --true = Debugging Mode
MiniPet.dprint = LibLordFarlander.dprint;
MiniPet.button = nil;

MiniPet.ReadyToInitialize    = false;

--DO NOT CHANGE THESE VALUES
MiniPet.lastCompanion        = nil;

MiniPet.showButtonImg       = false; --Goes on and off if button flashes
MiniPet.flashTime           = 0;     --Time since button flashes
MiniPet.flashButton         = false; --On when the button flashes

MiniPet.keyBoundMode        = false;

MiniPet.dirty               = false;
MiniPet.dirtyBags = { [-254] = true, [-255] = true, };
for bag = 0, NUM_BAG_SLOTS do
    MiniPet.dirtyBags[bag] = true;
end--for

function MiniPet:OnInitialize()
    local AceConfigReg = LibStub( "AceConfigRegistry-3.0" );
    local AceConfigDialog = LibStub( "AceConfigDialog-3.0" );
    local LibLordFarlander_UI = LibStub( "LibLordFarlander-UI-2.0" );
	local defaults = self:GetDefaults();
    local buttonOptions = LibLordFarlander_UI.GetButtonOptionGroup( self, {
            name = L["Button"],
            desc = L["Options relating to the button."],
            yes = L["yes"],
            no = L["no"],
            show = L["showing"],
            hide = L["hiding"],
            locked = L["locked"],
            unlocked = L["unlocked"],
            toggleButton = L["toggleButton"],
            toggleButton_name = L["Show button"],
            toggleButton_desc = L["Show/hide the button."],
            autoHideButton = L["autoHideButton"],
            autoHideButton_name = L["Automatically hide button"],
            autoHideButton_desc = L["Hide the button when you click it?"],
            lockButton = L["lockButton"],
            lockButton_name = L["Lock button position"],
            lockButton_desc = L["Lock/unlock the position of the button."],
            alpha = L["alpha"],
            alpha_name = L["Alpha of button"],
            alpha_desc = L["Sets the alpha value that the button has normally."],
            fadeOut = L["fadeOut"],
            fadeOut_name = L["Fade out"],
            fadeOut_desc = L["Fade the button out when the cursor is not over it?"],
            fadeOutInsteadHide = L["fadeOutInsteadHide"],
            fadeOutInsteadHide_name = L["Fade out instead of hiding"],
            fadeOutInsteadHide_desc = L["Fade the button out instead of hiding it?"],
            fadeOutCancelInCombat = L["fadeOutCancelInCombat"],
            fadeOutCancelInCombat_name = L["Cancel fade out if in combat"],
            fadeOutCancelInCombat_desc = L["Should the button not fade out if you're in combat?"],
            fadeOutCancelOnShift = L["fadeOutCancelOnShift"],
            fadeOutCancelOnShift_name = L["Cancel fade out if holding Shift"],
            fadeOutCancelOnShift_desc = L["Should the button not fade out if you're holding Shift?"],
            fadeOutCancelOnCtrl = L["fadeOutCancelOnCtrl"],
            fadeOutCancelOnCtrl_name = L["Cancel fade out if holding Ctrl"],
            fadeOutCancelOnCtrl_desc = L["Should the button not fade out if you're holding Ctrl?"],
            fadeOutCancelOnAlt = L["fadeOutCancelOnAlt"],
            fadeOutCancelOnAlt_name = L["Cancel fade out if holding Alt"],
            fadeOutCancelOnAlt_desc = L["Should the button not fade out if you're holding Alt?"],
            fadeOutTime = L["fadeOutTime"],
            fadeOutTime_name = L["Fade out time"],
            fadeOutTime_desc = L["Sets how long the button takes to fade out."],
            fadeOutDelay = L["fadeOutDelay"],
            fadeOutDelay_name = L["Fade out delay"],
            fadeOutDelay_desc = L["Sets the delay before the button starts to fade."],
            fadeOutAlpha = L["fadeOutAlpha"];
            fadeOutAlpha_name = L["Alpha after fadeout"],
            fadeOutAlpha_desc = L["Sets the alpha value that the button has after faded out."],
            buttonScale = L["buttonScale"],
            buttonScale_name = L["Scale of button"],
            buttonScale_desc = L["Sets the scaling value for the button to make it bigger or smaller."],
        }, 1 );
    local options = {
        type = "group",
        handler = self,
        args = self:GetModuleArgs(),
        childGroups = "tree",
        name = L["MiniPet"],
        desc = L["Addon options."],
        icon = [[Interface\AddOns\MiniPet\Img\MiniPet]],
    };
    local additionalOptions = self:GetAdditionalOptionTables();

    self.db = LibStub( "AceDB-3.0" ):New( "MiniPetDB", defaults );
    self.db.RegisterCallback( self, "OnProfileChanged" );
    self.db.RegisterCallback( self, "OnProfileReset" );

    local profileOptions = LibStub( "AceDBOptions-3.0" ):GetOptionsTable( self.db );
    local slashOptions = {
        type = "group",
        handler = self,
        name = options.name,
        desc = options.desc,
        icon = options.icon,
        args = {
            [L["options"]] = options,
            [L["favorites"]] = favoritesOptions,
            [L["zoneFavorites"]] = zoneOptions,
            [L["button"]] = buttonOptions,
            [L["profile"]] = profileOptions,
            [L["gui"]] = {
                type = "execute",
                order = -1,
                name = L["Open GUI configurator"],
                desc = L["Open GUI configurator"],
                guiHidden = true,
                func = function( info )
                           InterfaceOptionsFrame_OpenToCategory( L[info.handler.name] );
                       end,
            },
            [L["newRandom"]] = {
                type = "execute",
                order = 6,
                name = L["Force the picking of a new random companion"],
                desc = L["Force the picking of a new random companion"],
                guiHidden = true,
                func = function( info )
                           info.handler:ChooseRandomCompanion();
                           info.handler:SetButton( info.handler:GetStateBasedOnModifiers() );
                       end,
            },
            [L["nextItem"]] = {
                type = "execute",
                order = 7,
                name = L["Cycle to the next companion modifier item"],
                desc = L["Cycle to the next companion modifier item"],
                guiHidden = true,
                func = function( info )
                           info.handler:DoCustomEvent( "FindNextCompanionEnchant" );
                           info.handler:SetButton( info.handler:GetStateBasedOnModifiers() );
                       end,
            },
            [L["summon"]] = {
                type = "execute",
                order = 8,
                name = L["Summons the current random companion (if a learned companion)"],
                desc = L["Summons the current random companion (if a learned companion)"],
                guiHidden = true,
                func = "SummonCompanion",
            },
            [L["dismiss"]] = {
                type = "execute",
                order = 9,
                name = L["Dismisses the current companion (if a learned companion)"],
                desc = L["Dismisses the current companion (if a learned companion)"],
                guiHidden = true,
                func = "DismissCompanion",
            },
            [L["list"]] = {
                type = "execute",
                order = 10,
                name = L["List all companions"],
                desc = L["Show a list with all of your companions."],
				func = "ListCompanions",
            },
            [L["refresh"]] = {
                type = "execute",
                order = 11,
                name = L["Refresh companion list"],
                desc = L["Force a refresh of the list of all your companions."],
                func = "ResetCompanions",
            },
            debug = {
                type = "toggle",
                hidden = true,
                name = "Debug",                   
                arg = {
                    DescriptionString = "Debug mode toggle",
                    VariableName = "DEBUG",
                    Location = "base",
                },
                desc = LibLordFarlander_UI.BooleanOptionDescription,
                get = LibLordFarlander_UI.GetGenericOption,
                set = LibLordFarlander_UI.SetGenericOption,                      
            },
        },
    };
    
    LibStub( "AceConfig-3.0" ):RegisterOptionsTable( L["MiniPet"], slashOptions, { L["minipet"], L["mp"] } );
    AceConfigReg:RegisterOptionsTable( "MiniPet Main Options", options );
    AceConfigReg:RegisterOptionsTable( "MiniPet Button", buttonOptions );
    AceConfigReg:RegisterOptionsTable( "MiniPet Profile", profileOptions );
    AceConfigDialog:AddToBlizOptions( "MiniPet Main Options", L["MiniPet"] );
    AceConfigDialog:AddToBlizOptions( "MiniPet Button", L["Button"], L["MiniPet"] );
    AceConfigDialog:AddToBlizOptions( "MiniPet Profile", L["Profile"], L["MiniPet"] );
    
    for name, options in pairs( additionalOptions ) do
        AceConfigReg:RegisterOptionsTable( "MiniPet " .. name, options );
        AceConfigDialog:AddToBlizOptions( "MiniPet " .. name, options.name, L["MiniPet"] );
    end--for
    
    if( LibStub( "LibDataBroker-1.1", true ) ) then
        self.ldbobj = LibStub( "LibDataBroker-1.1" ):NewDataObject( L["MiniPet"], {
                type = "data source",
                icon = [[Interface\AddOns\MiniPet\Img\MiniPet]],
                label = L["MiniPet"],
                text = L["Get a companion for some company!"],
                secureTemplate = "SecureActionButtonTemplate",
                OnClick = function( button, btn )
                              if( button:IsObjectType( "SecureActionButtonTemplate" ) ) then
                                  self.button:Click( btn );
                              else
                                  if( not self:DoCustomEventWithReturn( "MouseClick", mouse, false ) ) then
                                      self:SummonCompanion();
                                  end--if
                              end--if
                          end,
                OnTooltipShow = function( tooltip )
                                    if( not tooltip ) then
                                        return;
                                    end--if
                                    MiniPet:SetTooltip( tooltip );
                                end,
            } );
    end--if
end--MiniPet:OnInitialize()

function MiniPet:OnProfileReset()
    self:OnProfileChanged();
end--MiniPet:OnProfileReset()

function MiniPet:OnProfileChanged()
    LibLordFarlander_UI.OnProfileChanged( self );
    if( self.db.profile.enchantOrder ) then
        local iterator, handle = PetAndMountDatabase:IterateSet( "Items.CritterEnchant" );
        local item, value = iterator( handle );

        while item do
            local inList = false;

            for _, listItem in pairs( self.db.profile.enchantOrder ) do
                if( listItem == item ) then
                    inList = true;
                    break;
                end--if
            end--for
            if( not inList ) then
                tinsert( self.db.profile.enchantOrder, item );
            end--if
            item, value = iterator( handle );
        end--for
    else
        self.db.profile.enchantOrder = TR:GetTable();

        local iterator, handle = PetAndMountDatabase:IterateSet( "Items.CritterEnchant" );
        local item, value = iterator( handle );
        local found = false;
        local counter = 1;

        while item do
            if( type( value ) == "boolean" ) then
                value = counter;
            else
                value = tonumber( value );
            end--if
            self.db.profile.enchantOrder[value] = item;
            item, value = iterator( handle );
            counter = counter + 1;
        end--for
    end--if
    --Upgrade check
    if( self.db.profile.FirstSeen ) then
        self.db.char.FirstSeen = self.db.profile.FirstSeen;
        self.db.profile.FirstSeen = nil;
    end--if
    if( ( self.db.profile.LastVersion == nil ) and ( ( MiniPet_BattleGroundCheck ~= nil ) or ( MiniPet_GiveTextWarning ~= nil ) or ( MiniPet_GiveButtonWarning ~= nil ) or ( MiniPet_HideButton ~= nil ) ) ) then
        --Upgrading from 2.0 or earlier, import the old settings
        self.db.profile.BattleGroundCheck = MiniPet_BattleGroundCheck;
        self.db.profile.GiveTextWarning = MiniPet_GiveTextWarning;
        self.db.profile.GiveButtonWarning = MiniPet_GiveButtonWarning;
        self.db.profile.Button.AutoHideButton = MiniPet_HideButton;
    end--if
    if( type( self.db.profile.LastVersion ) == "number" ) then
        --Upgrading from pre-3.5, need to convert zone favorites to the new format
        local newList = TR:GetTable();

        for zone, itemID in pairs( self.db.profile.ZoneFavorites ) do
            newList[zone] = { itemID = { exclusive = true, } };
        end--for
        self.db.profile.ZoneFavorites = newList;
    end--if
    if( ( type( self.db.profile.LastWoWVersion ) ~= "number" ) or ( self.db.profile.LastWoWVersion < 30000 ) ) then
        --Upgrading from pre-wotlk, need to convert itemIDs in favorites and zone favorites to spellIDs and to the new format
        local newID;
        local newList = TR:GetTable();

        for itemID, value in pairs( self.db.profile.Favorites ) do
            newID = PetAndMountDatabase:ItemInSet( itemID, "Conversion.Critters" );
            if( newID ) then
                itemID = tonumber( newID );
            end--if
            newList[itemID] = ( type( value ) == "number" ) and value or 2.0;
            if( newList[itemID] == 1.0 ) then
                newList[itemID] = nil;
            end--if
        end--for
        self.db.profile.Favorites = newList;
        newList = TR:GetTable();
        for zone, pets in pairs( self.db.profile.ZoneFavorites ) do
            newList[zone] = TR:GetTable();
            for itemID, info in pairs( pets ) do
                newID = PetAndMountDatabase:ItemInSet( itemID, "Conversion.Critters" );
                if( newID ) then
                    itemID = tonumber( newID );
                end--if
                newList[zone][itemID] = info;
                if( not ( info.frequency or info.active or info.exclusive ) ) then
                    newList[zone][itemID] = nil;
                end--if                
            end--for
        end--for
        self.db.profile.ZoneFavorites = newList;
    end--if
    self.db.profile.LastWoWVersion = LibLordFarlander.GetTOCVersion();
    self.db.profile.LastVersion = self.version;

    self.dirty = true;
    --Create the Pet Inventory
    self:GetPlayersCompanions();
    self:BuildZoneFavorites();
    --Set a Pet
    self:ChooseRandomCompanion();
    self:SetButton( self:GetStateBasedOnModifiers() );
end--MiniPet:OnProfileChanged()

function MiniPet:OnEnable()
    --Register the Events
    self:RegisterEvent( "BAG_UPDATE",
        function( _, bagID )
            if( bagID and ( bagID >= 0 ) and ( bagID <= NUM_BAG_SLOTS ) ) then
                self:DirtyBag( bagID );
            end--if
        end );
    self:RegisterEvent( "MODIFIER_STATE_CHANGED",
        function()
            self:SetButton( self:GetStateBasedOnModifiers() );
        end );

    LibLordFarlander_UI.SetupForUI( self );
    self.button = LibLordFarlander_UI.CreateButton( self, self.name, [[Interface\AddOns\MiniPet\Img\MiniPet]], self.db.profile.ButtonGroup, self.SkinChanged, self );
    ItemList.AttachToButton( self.button );
    self.button:SetMovable( not self.db.profile.Button.LockButton );
    self.button:HookScript( "OnClick",
        function( self, mouse )
            local class = self.class;
            
            class.flashButton = false; --Stop Flashing the Button :)
            if( class.db.profile.Button.AutoHideButton ) then
                self:HideEx();
            end--if
            if( not class:DoCustomEventWithReturn( "MouseClick", mouse, true ) ) then
                class:ChooseRandomCompanion();
                class:SetButton( class:GetStateBasedOnModifiers() );
            end--if
        end );

    self:OnProfileChanged();
end--MiniPet:OnEnable();

function MiniPet:OnDisable()
end--MiniPet:OnDisable()

function MiniPet:OnUpdate( elapsed )
    if( ( UnitName( "player" ) ~= UNKNOWNOBJECT ) and self.readyToInitialize ) then
        self:DoCustomEvent( "EnteringWorld" );
        self.readyToInitialize = false;
    end--if
    
    self:DoCustomEvent( "Update" );

    if( self.dirty ) then
        self.dirty = false;
        if( self:GetPlayersCompanions() ) then
            self:BuildZoneFavorites();
            self:ChooseRandomCompanion();
        end--if
        self:SetButton( self:GetStateBasedOnModifiers() );
    end--if

    --Code to let the button flash
    --elapsed = amount of seconds that passed since last update.
    --every updateinterval, change the button image
    --until you reach totalflashtime
    --should be made a combination of MiniPet UI functions
    --and stuff in a Warnings module
    if( self.flashButton and elapsed ) then
        local mpButton = self.button;

        if( mpButton.TimeSinceLastUpdate == nil ) then
            mpButton.TimeSinceLastUpdate = 0;
            self.flashTime = 0;
        end--if
        mpButton.TimeSinceLastUpdate = mpButton.TimeSinceLastUpdate + elapsed;
        self.flashTime = self.flashTime + elapsed;
        if( self.flashTime < self.db.profile.TotalFlashTime ) then
            if( mpButton.TimeSinceLastUpdate > self.db.profile.UpdateInterval ) then
                local currentCompanion = self:GetStateBasedOnModifiers();
                
                if( self.showButtonImg and ( currentCompanion ~= nil ) ) then
                    mpButton:SetFromListEntry( currentCompanion );
                    self.showButtonImg = false;
                else
                    mpButton:SetImage( [[Interface\AddOns\MiniPet\Img\MiniPet]] );
                    self.showButtonImg = true;
                end--if
                mpButton.TimeSinceLastUpdate = 0;
            end--if
        else
            self.flashButton = false;
            self:SetButton( self:GetStateBasedOnModifiers() );
            self.flashTime = 0;
        end--if
    end--if
end--MiniPet:OnUpdate( elapsed )

function MiniPet:WarnPlayer()
    return self:DoCustomEventWithReturn( "AlertPlayer" );
end--MiniPet:WarnPlayer()

function MiniPet:CombatStarted()
    self:DoCustomEvent( "StartCombat" );
end--MiniPet:CombatStarted()

function MiniPet:CombatEnded()
    self.button:OutOfCombatReset();
    self:SetButton( self:GetStateBasedOnModifiers() );
    self:DoCustomEvent( "EndCombat" );
end--MiniPet:CombatEnded()

function MiniPet:DirtyBag( bag )
    self.dirtyBags[bag] = true;
    self.dirty = true;
    if( self.button and ( not self.button:IsVisible() ) ) then
        self:ScheduleTimer( "OnUpdate", 0 );
    end--if
end--MiniPet:DirtyBag( bag )

----------------------------
--+Pet/Inventory Functions--
function MiniPet:GetStateBasedOnModifiers()
    return self:DoCustomEventWithReturn( "GetCurrentCompanion" );
end--MiniPet:GetStateBasedOnModifiers()

--Pick a random pet and set the button.
function MiniPet:ChooseRandomCompanion()
    self:DoCustomEvent( "ChooseRandom" );
end--MiniPet:ChooseRandomCompanion()

function MiniPet:GetAllCompanionsList( indexed )
    return self:DoCustomEventWithReturn( "GetAllCompanionsList", indexed );
end--MiniPet:GetAllCompanionsList()

--Gets all your pets from your inventory.
function MiniPet:GetPlayersCompanions()
    return self:ScanPlayer();
end--MiniPet:GetPlayersCompanions()

function MiniPet:DismissCompanion()
    self:DoCustomEvent( "DismissCompanion" );
end--MiniPet:DismissCompanion()

function MiniPet:SummonCompanion()
    self:DoCustomEvent( "SummonCompanion" );
end--MiniPet:SummonCompanion()

function MiniPet:ListCompanions( info )
    local companions = self:GetAllCompanionsList();
    local zoneList = self:GetZoneList();
    local curTime = time();
    local db = self.db;
    local profile = db.profile;
    local firstSeen = db.char.FirstSeen;
    local boostNew = profile.BoostNew;
    local boostNewTime = profile.BoostNewTime;  

    self:Print( L["List of your companions [%i total]:"]:format( #companions ) );
    table.sort( companions, function( a, b ) return a:GetName() < b:GetName() end );
    for _, companion in pairs( companions ) do  
        local itemID = companion:GetID();
        local howOften = zoneList[itemID] or 0.0;
        
        if( ( howOften > 0.0 ) and boostNew and firstSeen[itemID] and ( curTime - firstSeen[itemID] <= boostNewTime ) ) then
            howOften = howOften + 20.0;
        end--if    
       self:Print( ("|T%s:0|t %s [%.1f%%]"):format( companion:GetTexture(), companion:GetLink(), howOften * 100.0 ) );
    end--for
    TR:ReturnTable( table.wipe( companions ) );
end--MiniPet:ListCompanions()

function MiniPet:ResetCompanions( info )
    for bag = 0, NUM_BAG_SLOTS do
        info.handler.dirtyBags[bag] = true;
    end--for
    info.handler.dirtyBags[-254] = true;
    info.handler.dirtyBags[-255] = true;
    info.handler:GetPlayersCompanions();
    info.handler:BuildZoneFavorites();
    info.handler:ChooseRandomCompanion();
    info.handler:SetButton( info.handler:GetStateBasedOnModifiers() );
end--MiniPet:ResetCompanions()

function MiniPet:BuildZoneFavorites()
    return self:DoCustomEventWithReturn( "BuildZoneFavorites" );
end--MiniPet:BuildZoneFavorites()

function MiniPet:GetZoneList()
    return self:DoCustomEventWithReturn( "GetZoneList" );
end--MiniPet:GetZoneList()
---Pet/Inventory Functions--
----------------------------

-------------------
--+Help Functions--
function MiniPet:SetOption_ResetPets( info, val )
    local arg = info.arg;

    self.db.profile[info.arg.VariableName] = val;
    self:GetPlayersCompanions();
    self:BuildZoneFavorites();
    self:ChooseRandomCompanion();
    self:SetButton( self:GetStateBasedOnModifiers() );
end--MiniPet:SetOption_ResetPets( info, val )

function MiniPet:BooleanOptionDescription( info )
    return LibLordFarlander_UI.BooleanOptionDescription( info );
end--MiniPet:BooleanOptionDescription( info )

function MiniPet:GetGenericOption( info )
    return LibLordFarlander_UI.GetGenericOption( info );
end--MiniPet:GetGenericOption( info )

function MiniPet:SetGenericOption( info, value )
    LibLordFarlander_UI.SetGenericOption( info, value );
end--MiniPet:SetGenericOption( info, value )

function MiniPet:GetSetting( section, setting, subsection )
    if( self.db[section] ) then
        if( subsection ) then
            if( self.db[section][subsection] ) then
                return self.db[section][subsection][setting];
            end--if
        else
            return self.db[section][setting];
        end--if
    end--if
    return nil;
end--Minipet:GetSetting( section, setting, subsection )

function MiniPet:SetSetting( section, setting, value, subsection )
    if( not self.db[section] ) then
        self.db[section] = {};
    end--if
    if( subsection ) then
        if( not self.db[section][subsection] ) then
            self.db[section][subsection] = {};
        end--if
        self.db[section][subsection][setting] = value;
    else
        self.db[section][setting] = value;
    end--if
end--Minipet:GetSetting( section, setting )
---Help Functions--
-------------------

-----------------
--+UI Functions--
function MiniPet.wprint( msg )
    MinipetTextFrameText:SetTextColor( 1.0, 0.1, 0.1 );
    MinipetTextFrameText:SetText( msg );
    MinipetTextFrame.startTime = GetTime();
    MinipetTextFrame:Show();
end--MiniPet.wprint( msg )

function MiniPet:SetTooltip( tooltip )
    local pet = self:GetStateBasedOnModifiers();

    if( pet == nil ) then
        tooltip:SetText( L["Get a companion for some company!"] );
    else
        ItemList.SetTooltipFromListEntry( tooltip, pet );
    end--if
end--Minipet:SetTooltip( tooltip )

function MiniPet:SetGameTooltips()
    local mpButton = self.button;

    if( not self.keyBoundMode ) then
        local mpTooltip = mpButton.tooltip;

        mpTooltip:SetOwner( mpButton, "RIGHT" );
        self:SetTooltip( mpTooltip );
    end--if
    LibLordFarlander_UI.AfterSetTooltip( mpButton );
end--MiniPet:SetGameTooltips()

function MiniPet:SetButton( critter )
    if( not InCombatLockdown() ) then
        local mpButton = self.button;
        local ldbobj = self.ldbobj;

        if( critter == nil ) then
            mpButton.cooldown:Hide();
            mpButton:SetAttribute( "type1", "item" );
            mpButton:SetAttribute( "*bag1", nil );
            mpButton:SetAttribute( "*slot1", nil );
            mpButton:SetAttribute( "*item1", nil );
            mpButton:SetImage( [[Interface\AddOns\MiniPet\Img\MiniPet]] );

            ldbobj.icon = [[Interface\AddOns\MiniPet\Img\MiniPet]];
            ldbobj.text = L["Get a companion for some company!"];
        else
            mpButton:SetFromListEntry( critter, not LibLordFarlander.CompareTables( self.lastCompanion, critter ) );
        end--if
        self.lastCompanion = critter;
    end--if
end--MiniPet:SetButton( pet )
---UI Functions--
-----------------

LibStub( "LibLordFarlander-Randomizer-2.0" ):RegisterHandler( "MiniPet", MiniPet );
