--[[
 Mount support module for MiniPet
 Version 5.1.219
 $Revision: 183 $

 All credit for authoring this addon is listed below
]]--

--[[
Copyright (c) 2010, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local L = LibStub( "AceLocale-3.0" ):GetLocale( "MiniPet mount support Module" );

local LibLordFarlander_UI = LibStub( "LibLordFarlander-UI-2.0", true );
if( not LibLordFarlander_UI ) then
    return;
end--if

if( not LibStub( "LibLordFarlander-Randomizer-2.0", true ) ) then
    return;
end--if

local Mount = {
    displayname = L["Mount"],
    settings = {
		[L["toggleMountWarning"]] = {
			type = "toggle",
			name = L["Mount warning"],                     
			arg = {
				DescriptionString = L["Show an alert when you dismount from your mount."],
				VariableName = "GiveMountWarning",
				Default = true,
			},
			desc = LibLordFarlander_UI.BooleanOptionDescription,
			get = LibLordFarlander_UI.GetGenericOption,
			set = LibLordFarlander_UI.SetGenericOption, 
		},
		[L["toggleFlyingMountWarning"]] = {
			type = "toggle",
			name = L["Only flying warning"],                     
			arg = {
				DescriptionString = L["Show an alert only when you dismount from a flying mount."],
				VariableName = "OnlyGiveFlyingMountWarning",
				Default = false,
			},
			desc = LibLordFarlander_UI.BooleanOptionDescription,
			get = LibLordFarlander_UI.GetGenericOption,
			set = LibLordFarlander_UI.SetGenericOption, 
		},        
    },
    events = {
        "UPDATE_SHAPESHIFT_FORM",
        "LibLordFarlander-SpecialEvents-Mount-4.0:Dismounted",
    },
};

function Mount:OnUPDATE_SHAPESHIFT_FORM( MiniPet )
    if( MiniPet:GetSetting( "profile", "GiveMountWarning" ) and ( select( 2, UnitClass( "player" ) ) == "Druid" ) and ( select( GetShapeshiftFormInfo( 1, GetShapeshiftForm( true ) ) ):find( "Ability_Druid_FlightForm", 1, true ) ~= nil ) ) then
        MiniPet:dprint( "Activating warning after flight form" );
        MiniPet:WarnPlayer();
    end--if
end--Mount:OnUPDATE_SHAPESHIFT_FORM( MiniPet )

function Mount:OnDismounted( MiniPet, _, _, flying )
    if( MiniPet:GetSetting( "profile", "GiveMountWarning" ) and
        ( flying or ( not MiniPet:GetSetting( "profile", "OnlyGiveFlyingMountWarning" ) ) ) ) then
        MiniPet:dprint( "Activating warning after mount" );
        MiniPet:WarnPlayer();
    end--if
end--Mount:OnDismounted( MiniPet, _, _, flying )

LibStub( "LibLordFarlander-Randomizer-2.0" ):RegisterModuleToHandler( "MiniPet", "Mount", Mount, { "Companions", "Warnings" } );
