--[[
 Warnings module for MiniPet
 Version 5.1.219
 $Revision: 183 $

 All credit for authoring this addon is listed below
]]--

--[[
Copyright (c) 2010, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local L = LibStub( "AceLocale-3.0" ):GetLocale( "MiniPet warnings Module" );

local LibLordFarlander_UI = LibStub( "LibLordFarlander-UI-2.0", true );
if( not LibLordFarlander_UI ) then
    return;
end--if

local PetAndMountDatabase = LibStub( "LibLordFarlander-PetAndMountDatabase-1.1", true );
if( not PetAndMountDatabase ) then
    return;
end--if

if( not LibStub( "LibLordFarlander-Randomizer-2.0", true ) ) then
    return;
end--if

local Warnings = {
    displayname = L["Warnings"],
    events = {
        "AlertPlayer",
        "Update",
        "EnteringWorld",
        "COMBAT_LOG_EVENT_UNFILTERED",
        "PLAYER_ALIVE",
        "PLAYER_DEAD",
        "PLAYER_ENTERING_WORLD",
        "PLAYER_UNGHOST",    
    },
    settings = {
        [L["toggleTextWarning"]] = {
            type = "toggle",
            order = 1,
            name = L["Text warning"],
            arg = {
                DescriptionString = L["Show text warnings to alert you to summon your companion."],
                VariableName = "GiveTextWarning",
                Default = true,
            },
            desc = LibLordFarlander_UI.BooleanOptionDescription,
            get = LibLordFarlander_UI.GetGenericOption,
            set = LibLordFarlander_UI.SetGenericOption,
        },
        [L["toggleButtonWarning"]] = {
            type = "toggle",
            order = 2,
            name = L["Button warning"],
            arg = {
                DescriptionString = L["Flash the button to alert you to summon your companion."],
                VariableName = "GiveButtonWarning",
                Default = true,
            },
            desc = LibLordFarlander_UI.BooleanOptionDescription,
            get = LibLordFarlander_UI.GetGenericOption,
            set = LibLordFarlander_UI.SetGenericOption,
        },
        ["updateInterval"] = {
            type = "range",
            order = 3,
            name = "Update interval",
            arg = {
                VariableName = "UpdateInterval",
                Default = 0.2,
            },
            guiHidden = true,
            dialogHidden = true,
            dropdownHidden = true,
            cmdHidden = true,
        },           
        ["totalFlashTime"] = {
            type = "range",
            order = 4,
            name = "Total flash time",
            arg = {
                VariableName = "TotalFlashTime",
                Default = 10,
            },
            guiHidden = true,
            dialogHidden = true,
            dropdownHidden = true,
            cmdHidden = true,
        },           
        [L["toggleBG"]] = {
            type = "toggle",
            order = 5,
            name = L["Battleground check"],
            arg = {
                DescriptionString = L["Show companion alerts while in a battleground."],
                VariableName = "BattleGroundCheck",
                Default = true,
            },
            desc = LibLordFarlander_UI.BooleanOptionDescription,
            get = LibLordFarlander_UI.GetGenericOption,
            set = LibLordFarlander_UI.SetGenericOption,
        },
        [L["delayWarnings"]] = {
            type = "toggle",
            order = 6,
            name = L["Delay warnings"],
            arg = {
                DescriptionString = L["Delay warnings until after combat or casting."],
                VariableName = "DelayWarnings",
                Default = true,
            },
            desc = LibLordFarlander_UI.BooleanOptionDescription,
            get = LibLordFarlander_UI.GetGenericOption,
            set = LibLordFarlander_UI.SetGenericOption,
        },        
    },
    required = true,
    playerState = {
        boardedTaxi      = false,
        startedTeleport  = false,
        teleportComplete = false,
        dead             = false,
        enteringWorld    = true,
    },
};

function Warnings:OnCOMBAT_LOG_EVENT_UNFILTERED( MiniPet, _, _, event, sourceGUID, _, _, _, _, _, spellID )
    if( ( sourceGUID == UnitGUID( "player" ) ) and event:find( "SPELL_" ) ) then
        if( event == "SPELL_CAST_START" ) then
            MiniPet:dprint( "Spellcast: " .. spellID );
            self.playerState.startedTeleport = false;
            if( PetAndMountDatabase:SpellInSet( spellID, "Spells.Teleport" ) ) then
                MiniPet:dprint( "Started Teleporting" );
                self.playerState.startedTeleport = true;
            end--if
        elseif( ( event == "SPELL_CAST_SUCCESS" ) and self.playerState.startedTeleport and PetAndMountDatabase:SpellInSet( spellID, "Spells.Teleport" ) ) then
                MiniPet:dprint( "Finished Teleporting" );
                self.playerState.startedTeleport = false;
                self.playerState.teleportComplete = true;
                self:WarnPlayer( MiniPet );
        elseif( ( event == "SPELL_CAST_FAILED" ) and self.playerState.startedTeleport ) then
                MiniPet:dprint( "Interrupted Teleporting" );
                self.playerState.startedTeleport = false;
        end--if
    end--if
end--Warnings:OnCOMBAT_LOG_EVENT_UNFILTERED( MiniPet, _, _, event, sourceGUID, _, _, _, _, _, spellID )

function Warnings:OnPLAYER_ALIVE( MiniPet )
    self.playerState.boardedTaxi = UnitOnTaxi( "player" );
    if( self.playerState.teleportComplete ) then
        self.playerState.teleportComplete = false;
        MiniPet:dprint( "Player Finished Hearth" );
        self:OnAlertPlayer( MiniPet );
    end--if
    if( self.playerState.dead and ( not UnitIsDeadOrGhost( "player" ) ) ) then
        self.playerState.dead = false;
        MiniPet:dprint( "Player No Longer Dead" );
        self:OnAlertPlayer( MiniPet );
    end--if
end--Warnings:OnPLAYER_ALIVE( MiniPet )

function Warnings:OnPLAYER_DEAD( MiniPet )
    MiniPet:dprint( "Player Is Dead" );
    self.playerState.dead = true;
end--Warnings:OnPLAYER_DEAD( MiniPet )

function Warnings:OnPLAYER_ENTERING_WORLD( MiniPet )
    self.playerState.dead = UnitIsDeadOrGhost( "player" );
    self.playerState.boardedTaxi = UnitOnTaxi( "player" );
    self.playerState.enteringWorld = false;    
    if( ( not self.playerState.dead ) and ( not self.playerState.boardedTaxi ) ) then
        MiniPet:dprint( "Player Login" );
        self:OnAlertPlayer( MiniPet );
    end--if
end--Warnings:OnPLAYER_ENTERING_WORLD( MiniPet )

function Warnings:OnPLAYER_UNGHOST( MiniPet )
    if( self.playerState.dead ) then
        self.playerState.dead = false;
        MiniPet:dprint( "Player No Longer Ghost" );
        self:OnAlertPlayer( MiniPet );
    end--if
end--Warnings:OnPLAYER_UNGHOST( MiniPet )

function Warnings:OnUpdate( MiniPet )
    if( self.playerState.boardedTaxi and ( not UnitOnTaxi( "player" ) ) ) then
        MiniPet:dprint( "Player No Longer On Taxi" );
        self.playerState.boardedTaxi = UnitOnTaxi( "player" );
        self:OnAlertPlayer( MiniPet );
    end--if
    if( UnitOnTaxi( "player" ) and ( not self.playerState.boardedTaxi ) ) then
        MiniPet:dprint( "Player Boarded Taxi" );
    end--if
    self.playerState.boardedTaxi = UnitOnTaxi( "player" );
end--Warnings:OnUpdate( MiniPet )

function Warnings:OnEnteringWorld( MiniPet )
    MiniPet:dprint( "EnteringWorld and ALIVE" );
    self.playerState.enteringWorld = false;
    self.ReadyToInitialize = false;
    MiniPet:dprint( "Activating Pet After Login [2]" );
    self:OnAlertPlayer();
end--Warnings:OnEnteringWorld( MiniPet )

function Warnings:OnAlertPlayer( Minipet )
    if( ( not ( self.IsInBattleground() and MiniPet:GetSetting( "profile", "BattleGroundCheck" ) ) ) and
        ( not self.playerState.dead ) and ( not self.playerState.boardedTaxi ) ) then--BG + option check
        if( MiniPet:GetSetting( "profile", "GiveTextWarning" ) and MiniPet:DoCustomEventWithReturn( "HasCompanions" ) ) then    --No use if player isn't cool and gots no pets!
            Minipet.wprint( L["ALERT: Time to summon your companion!"] );
        end--if
        -- TODO: should probably be moved to MiniPet
        Minipet.FlashButton = MiniPet:GetSetting( "profile", "GiveButtonWarning" );
        if( MiniPet:GetSetting( "profile", "AutoHideButton", "Button" ) and ( not Minipet.button:IsVisible() ) ) then
            Minipet.button:ShowEx();
        end--if
    end--if
    MiniPet:DoCustomEvent( "WarnPlayer" );    
end--Warnings::OnAlertPlayer( Minipet )

function Warnings.IsInBattleground()
    local isInstanced, itype = IsInInstance();

    return isInstanced and ( itype == "pvp" );
end--Warnings.IsInBattleground()

LibStub( "LibLordFarlander-Randomizer-2.0" ):RegisterModuleToHandler( "MiniPet", "Warnings", Warnings );
